/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 337 (Exercise B.1.1)
**
*/

new;
library rpb,pgraph;

cls;

let sigma = 0.15 0.15 0.05;
let mu =    0.10 0.10 0.05;
let rho = 1.00 
          0.50 1.00
          0.20 0.20 1.00;
rho = xpnd(rho);
cov = rho .* sigma .* sigma';

print;
print "=====================  Questions 1, 2 & 3 =======================================";

/*
**> Question 1
**
*/

gamma0 = 0;
A = ones(1,3); B = 1; C = 0; D = 0; Bounds = (0~1) .* ones(3,1);
{x0,sigma0,retcode} = Compute_MV_Portfolio(cov,A,B,C,D,Bounds);

/*
**> Question 2
**
*/

{x1,mu1,sigma1,gamma1,retcode1} = Compute_MVO_Portfolio(mu,cov,A,B,C,D,Bounds,0.05,2);

/*
**> Question 3
**
*/

{x2,mu2,sigma2,gamma2,retcode2} = Compute_MVO_Portfolio(mu,cov,A,B,C,D,Bounds,0.10,2);

phi0 = 1/gamma0; phi1 = 1/gamma1; phi2 = 1/gamma2; 

{x2_bis,u1,u2,u3,u4,retcode} = Qprog(ones(3,1)/3,cov,1/phi2*mu,A,B,C,D,Bounds);

print ftosa(100*(x0~x1~x2~x2_bis),"%lf",3,2);
print ftosa(100*(sigma0~sigma1~sigma2),"%lf",3,2);
print ftosa(phi0~phi1~phi2,"%lf",3,2);

/*
**> Question 5
**
*/

A = ones(1,3); B = 1; C = 0; D = 0; Bounds = (0~1) .* ones(3,1); Bounds[1,1] = 0.08;

print;
print "=====================  Questions 5.a =======================================";

{x0,sigma0,retcode} = Compute_MV_Portfolio(cov,A,B,C,D,Bounds);
{x1,mu1,sigma1,gamma1,retcode1} = Compute_MVO_Portfolio(mu,cov,A,B,C,D,Bounds,0.05,2);
{x2,mu2,sigma2,gamma2,retcode2} = Compute_MVO_Portfolio(mu,cov,A,B,C,D,Bounds,0.10,2);
phi0 = 1/gamma0; phi1 = 1/gamma1; phi2 = 1/gamma2; 

print ftosa(100*(x0~x1~x2),"%lf",3,2);
print ftosa(100*(sigma0~sigma1~sigma2),"%lf",3,2);
print ftosa(phi0~phi1~phi2,"%lf",3,2);

print;
print "=====================  Questions 5.c =======================================";

// Calculus of the Lagrange coefficient with the primal program

A = ones(1,3); B = 1; C = 1~0~0; D = 0.08; Bounds = (0~1) .* ones(3,1); 
{x0,u1,lagr0,u3,u4,retcode0} = QProg(x0,cov,gamma0*mu,A,B,C,D,Bounds);
{x1,u1,lagr1,u3,u4,retcode0} = QProg(x0,cov,gamma1*mu,A,B,C,D,Bounds);
{x2,u1,lagr2,u3,u4,retcode0} = QProg(x0,cov,gamma2*mu,A,B,C,D,Bounds);  

print ftosa(100*(lagr0~lagr1~lagr2),"%lf",3,4);

// Calculus of the Lagrange coefficient with the dual program

S = -C|A|-A|-eye(3); T = -D|B|-B|zeros(3,1);
Bounds = (0~10) .* ones(6,1);
Q_tilde = S*invpd(cov)*S';
R_tilde = gamma0*S*invpd(cov)*mu - T;
{lambda0,u1,u2,u3,u4,retcode0} = QProg(zeros(6,1),Q_tilde,R_tilde,0,0,0,0,Bounds);
R_tilde = gamma1*S*invpd(cov)*mu - T;
{lambda1,u1,u2,u3,u4,retcode0} = QProg(zeros(6,1),Q_tilde,R_tilde,0,0,0,0,Bounds);
R_tilde = gamma2*S*invpd(cov)*mu - T;
{lambda2,u1,u2,u3,u4,retcode0} = QProg(zeros(6,1),Q_tilde,R_tilde,0,0,0,0,Bounds);
print ftosa(100*(lambda0[1]~lambda1[1]~lambda2[1]),"%lf",3,4);

A = ones(1,3); B = 1; C = 0; D = 0; Bounds = (0~1) .* ones(3,1); Bounds[1,1] = 0.20;
{x0,sigma0,retcode} = Compute_MV_Portfolio(cov,A,B,C,D,Bounds);
print ftosa(100*sigma0,"Lower bound of the volatility = %lf",3,2);

print;
print "=====================  Questions 6 =======================================";

sigma1 = sigma[1]; sigma2 = sigma[2]; sigma3 = sigma[3];
rho12 = rho[1,2]; rho13 = rho[1,3]; rho23 = rho[2,3];
d = (sigma2^2 - 2*rho23*sigma2*sigma3 + sigma3^2);
a = (sigma3^2 - rho23*sigma2*sigma3)/d;
b = (sigma2^2 - rho23*sigma2*sigma3)/d;
c = (sigma1 *(rho12 * sigma2 - rho13 * sigma3))/d;
alpha = sigma1^2 + (a+c)^2 * sigma2^2 + (b-c)^2 * sigma3^2 - 2*(a+c)*rho12*sigma1*sigma2 -
        2*(b-c)*rho13*sigma1*sigma3 + 2*(a+c)*(b-c)*rho23*sigma2*sigma3;
_beta =  a*rho12*sigma1*sigma2 + b*rho13*sigma1*sigma3 - 
        (a*(b-c)+b*(a+c))*rho23*sigma2*sigma3 - a*(a+c)*sigma2^2 - b*(b-c)*sigma3^2;
gamma_ = a^2 * sigma2^2 + b^2 * sigma3^2 + 2*a*b*rho23*sigma2*sigma3;

sigma_star = 0.05;
xi = real(polyroot(alpha|2*_beta|(gamma_-sigma_star^2)));
print ftosa(100*xi,"%lf",3,5);

x_min = seqa(0,0.001,250);
sigma0 = zeros(rows(x_min),1);

i = 1;
do until i > rows(x_min);
  A = ones(1,3); B = 1; C = 0; D = 0; Bounds = (0~1) .* ones(3,1); Bounds[1,1] = x_min[i];
  {x0,sigma0[i],retcode} = Compute_MV_Portfolio(cov,A,B,C,D,Bounds);
  i = i + 1;
endo;

graphset;
  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6; _pcolor = 1; _plwidth = 10;
  xtics(0,20,2,2);
  xlabel("\216x]1[[-] (in %)");
  ytics(4.9,5.3,0.1,2);
  _pline = 1~3~0~5~25~5~1~0~10|
           1~3~100*xi[1]~4.8~100*xi[1]~6.0~1~0~10;
  _pmsgstr = "\202x\201[*]";
  _pmsgctl = 100*xi[1]+0.20~4.92~0.15~0~1~0~8;
  graphprt("-c=1 -cf=app2-1-1-1.eps");
  xy(100*x_min,100*sigma0);

