/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 374 (Exercise B.3.10)
**
*/

new;
library rpb,pgraph;

cls;

/*
**> Question 1
**
*/

print;
print "=====================  Question 1.a  =======================================";

r = 0.02;
let mu = 0.04 0.05 0.06 0.07;
let sigma = 0.10 0.15 0.20 0.25;
rho = eye(4);
cov = rho .* sigma .* sigma';
n = rows(cov);

e = ones(n,1);

c = 1 / (e'*invpd(cov)*(mu-r));
x_star0 = c * invpd(cov)*(mu-r);
b = ((mu-r)./sigma)^2;
b = b / sumc(b);

{x_rb0,vol,mr,rc,prc} = Compute_RB_Portfolio(0,cov,b);

print ftosa(100*(b~x_star0~x_rb0),"%lf",3,2);

SR = (mu-r)./sigma;

print;
print "=====================  Question 2.c  =======================================";

pi_bar = mu - r;
sigma_bar = sigma;
b_bar = b;

let mu = 0.03 0.04 0.07 0.08;
let sigma = 0.12 0.12 0.21 0.24;

cov = rho .* sigma .* sigma';
pi_ = mu - r;

params = pi_bar~sigma_bar~pi_~sigma;
print ftosa(100*params,"%lf",3,2);

c = 1 / (e'*invpd(cov)*(mu-r));
x_star1 = c * invpd(cov)*(mu-r);

theta = seqa(0,2/50,51);
nIters = rows(theta);
all_x = zeros(n,nIters);
all_b = zeros(n,nIters);

iter = 1;
do until iter > nIters;
  alpha = theta[iter]; _beta = alpha; gamma_ = alpha; delta = alpha;
  b = b_bar .* (pi_^alpha ./ pi_bar^_beta) .* (sigma_bar^gamma_ ./ sigma^delta);
  b = b / sumc(b);
  {x,vol,mr,rc,pc} = Compute_RB_Portfolio(0,cov,b);
  all_b[.,iter] = b;
  all_x[.,iter] = x;
  iter = iter + 1;
endo;

graphset;
  window3d;

    fonts("simplex simgrma");
    _pdate = ""; _pframe = 0; _pnum = 2;
    _pltype = 1|6|3|2; _plwidth = 10; _pcolor = 1|2|3|4;
    _paxht = 0.25; _pnumht = 0.20; _ptitlht = 0.275;
    
  setwind(1);
    xlabel("\216\202Q\201");
    ylabel("\216b]i[(t)");
    xtics(0,2,0.5,2);
    ytics(5,40,5,2);
    xy(theta,100*all_b');

  setwind(2);
    ylabel("\216x]i[(t)");
    ytics(15,35,5,2);
    xy(theta,100*all_x');

  setwind(3);
    xlabel("\216b]i[(t)");
    xtics(5,40,5,2);
    _pline = 1~1~25~15~25~35~1~0~7;
    _plegstr = "Asset #1\000Asset #2\000Asset #3\000Asset #4";
    _plegctl = {2 6 9 4};
    xy(100*all_b',100*all_x');

    graphprt("-c=1 -cf=app2-3-10-1.eps");

  endwind;

alpha = 0; _beta = 0; gamma_ = 0; delta = 0;
b = b_bar .* (pi_^alpha ./ pi_bar^_beta) .* (sigma_bar^gamma_ ./ sigma^delta);
b = b / sumc(b);
{x_rb1,vol,mr,rc,pc} = Compute_RB_Portfolio(0,cov,b);

print ftosa(100*(b~x_star0~x_rb0~x_star1~x_rb1),"%lf",3,2);

alpha = 2; _beta = 2; gamma_ = 2; delta = 2;
b = b_bar .* (pi_^alpha ./ pi_bar^_beta) .* (sigma_bar^gamma_ ./ sigma^delta);
b = b / sumc(b);
{x_rb1,vol,mr,rc,prc} = Compute_RB_Portfolio(0,cov,b);

print ftosa(100*(b~x_star0~x_rb0~x_star1~x_rb1),"%lf",3,2);

alpha = 0; _beta = -2; gamma_ = -4; delta = -2;
b = b_bar .* (pi_^alpha ./ pi_bar^_beta) .* (sigma_bar^gamma_ ./ sigma^delta);
b = b / sumc(b);
{x_rb1,vol,mr,rc,prc} = Compute_RB_Portfolio(0,cov,b);

print ftosa(100*(b~x_star0~x_rb0~x_star1~x_rb1),"%lf",3,2);

alpha = 0; _beta = 0; gamma_ = -2; delta = -2;
b = b_bar .* (pi_^alpha ./ pi_bar^_beta) .* (sigma_bar^gamma_ ./ sigma^delta);
b = b / sumc(b);
{x_rb1,vol,mr,rc,prc} = Compute_RB_Portfolio(0,cov,b);

print ftosa(100*(b~x_star0~x_rb0~x_star1~x_rb1),"%lf",3,2);

/*
**> Question 3
**
*/

PathName = "data\\";
datFileName = PathName $+ "msci-wgbi-djubs-fed"; 
data = loadd(datFileName);
vnames = "" $+ getname(datFileName);

Begin_Date = 19981231;
End_Date = 20111230;

data = selif(data,(data[.,1] .>= Begin_Date) .and (data[.,1] .<= End_Date));
data = Fill_Miss(data,4);

Dates = data[.,1];
nDates = rows(Dates);

Months = trunc(Dates/100);
RD_Monthly = Months ./= lag1(Months);
RD_Monthly[1] = 1;
RD_Weekly = zeros(nDates,1);
RD_Weekly[seqa(1,5,nDates/5)] = ones(nDates/5,1);
RD = RD_Weekly;

Basket = Data[.,2 3];
Basket = 100 * Basket ./ Basket[1,.];
nAssets = cols(Basket);
Libor = Data[.,5]/100;
Backtest_Libor = Capitalized_Libor(Dates,Libor);

print;
print "=====================  Question 3.a  =======================================";

short_cov_lags = 20;
cov_Lags = 260;
trend_lags = 260;

{short_cov,vol,cor} = Rolling_Covariance(Dates,Basket,short_cov_lags,1);
{cov,vol,cor} = Rolling_Covariance(Dates,Basket,cov_lags,1);
Trends = Basket ./ lagn(Basket,trend_lags) - 1.0;
RFR = Backtest_Libor ./ lagn(Backtest_Libor,trend_lags) - 1.0;
{indx_beg,indx_end} =  indn_nomiss(Trends);
indx_beg = indx_beg[1];
Trends = missrv(Trends,Trends[indx_beg,.]);
RFR = missrv(RFR,RFR[indx_beg,.]);

weights1 = miss(zeros(nDates,nAssets),0);
weights2 = miss(zeros(nDates,nAssets),0);
weights3 = miss(zeros(nDates,nAssets),0);

rc1 = miss(zeros(nDates,nAssets),0);
rc2 = miss(zeros(nDates,nAssets),0);
rc3 = miss(zeros(nDates,nAssets),0);

b1 = miss(zeros(nDates,nAssets),0);
b2 = miss(zeros(nDates,nAssets),0);
b3 = miss(zeros(nDates,nAssets),0);

x0 = ones(nAssets,1)/nAssets;
let b0 = 0.50 0.50;

t = 1;
do until t > nDates;

  cov_t = arraytomat(cov[_max_(t,cov_lags),.,.]);
  short_cov_t = arraytomat(short_cov[_max_(t,short_cov_lags),.,.]);
  mu_t = Trends[t,.]';
  r_t = RFR[t];
  sigma_t = sqrt(diag(cov_t));
  SR_t = mu_t ./ sigma_t;
  SR_t = _max_(-0.5,SR_t);
  SR_t = _min_(1.5,SR_t);

  if RD[t] == 1;

    {x,vol,mr,rc,prc} = Compute_ERC_Portfolio(x0,cov_t);
    weights1[t,.] = x';
    rc1[t,.] = prc';
    b1[t,.] = b0';

    b_t = b0 .* (1 + SR_t)^1;
    b_t = b_t / sumc(b_t);

    {x,vol,mr,rc,prc} = Compute_RB_Portfolio(x0,cov_t,b_t);
    weights2[t,.] = x';
    rc2[t,.] = prc';
    b2[t,.] = b_t';

    x = invpd(cov_t) * (mu_t - r_t);
    x = x / sumc(x);
    x = _min_(_max_(0,x),1.0);
    {vol,mr,rc,prc} = Compute_Risk_Contribution(x,cov_t);
    weights3[t,.] = x';
    rc3[t,.] = prc';

  else;

    x = weights1[t-1,.]';
    {vol,mr,rc,prc} = Compute_Risk_Contribution(x,cov_t);
    weights1[t,.] = x';
    rc1[t,.] = prc';
    b1[t,.] = b1[t-1,.];

    x = weights2[t-1,.]';
    {vol,mr,rc,prc} = Compute_Risk_Contribution(x,cov_t);
    weights2[t,.] = x';
    rc2[t,.] = prc';
    b2[t,.] = b2[t-1,.];

    x = weights3[t-1,.]';
    {vol,mr,rc,prc} = Compute_Risk_Contribution(x,cov_t);
    weights3[t,.] = x';
    rc3[t,.] = prc';
    b3[t,.] = b3[t-1,.];

  endif;

  t = t + 1;
endo;

Backtests = zeros(nDates,3);
nBacktests = cols(Backtests);
Backtests[.,1] = Simulate_Monthly_Backtest(Dates,Weights1,Basket,0,0);
Backtests[.,2] = Simulate_Monthly_Backtest(Dates,Weights2,Basket,0,0);
Backtests[.,3] = Backtest_Libor;

t_Daily = selif(seqa(1,1,nDates), Dates .>= 20000101);
t_Dates = Dates[t_Daily];
Backtests = Backtests[t_Daily,.];
Backtests = 100 * Backtests./Backtests[1,.];
Libor_Index = Backtests[.,cols(Backtests)];
t_RD = RD[t_Daily];
eq_budgets = b1[t_Daily,1]~b2[t_Daily,1];
eq_weights = weights1[t_Daily,1]~weights2[t_Daily,1]~weights3[t_Daily,1];

turnover = zeros(nBacktests,1);

i = 1;
do until i > nBacktests-1;
  str = itos("weights",i);
  x = varget(str);
  x = x[t_Daily,.];
  x = selif(x,t_RD);
  turnover[i] = 52*meanc(packr(sumr(abs(x - lag1(x)))));
  i = i + 1;
endo;

indx_t = selif(seqa(1,1,rows(t_Dates)),t_RD);
t = seqa(1,1,rows(indx_t));

pqgwin many;

graphset;
  window3u;

    _pdate = ""; _pnum = 2; _pframe = 0; 
    _pltype = 3|6|1; _plwidth = 10; _pcolor = 1|2|3;
    _paxht = 0.20; _pnumht = 0.20; _ptitlht = 0.275;

    xtics(1,rows(t),52.08*2,2);
    let xlab = "2000" "02" "04" "06" "08" "10" "12";
    asclabel(xlab,0);

  setwind(1);
    ytics(0,100,20,2);
    xy(t,100*eq_budgets[indx_t,1 2]);

  setwind(2);
    ytics(0,100,20,2);
    xy(t,100*eq_weights[indx_t,1 2]);

  setwind(3);
    ytics(80,220,20,2);
    _plegstr = "Risk parity\000Dynamic risk parity\000Fed fund";
    _plegctl = {2 6 1.5 5.25};
    xy(t,backtests[indx_t,1 2]);

    graphprt("-c=1 -cf=app2-3-10-2.eps");
  endwind;

stats = zeros(6,nBacktests);

iter = 1;
do until iter > nBacktests;
  report = Reporting_Backtest(t_Dates[.,1],Backtests[.,iter],0,Libor_Index,0,0);
  stats[1,iter] = vread(Report,"mu");
  stats[2,iter] = vread(Report,"sigma");
  stats[3,iter] = vread(Report,"SR");
  stats[4,iter] = vread(Report,"MDD");
  stats[5,iter] = vread(Report,"sk");
  stats[6,iter] = vread(Report,"kt")-3.0;
  iter = iter + 1;
endo;

stats[1:2 4,.] = stats[1:2 4,.] * 100;
stats = stats|turnover';
print ftosa(stats',"%lf",3,2);

print;
print "=====================  Question 3.b  =======================================";

graphset;
  begwind;
  window(2,2,0);

  _pdate = ""; _pnum = 2; _pframe = 0; 
  _pltype = 6; _plwidth = 10; _pcolor = 1|2|3;
  _paxht = 0.20; _pnumht = 0.20; _ptitlht = 0.275;

  xtics(1,rows(t),52.08*2,2);
  let xlab = "2000" "02" "04" "06" "08" "10" "12";
  asclabel(xlab,0);
  ytics(0,100,20,2);
  ylabel("\216Weights (in %)");

  setwind(1);
    title("\216Dynamic risk budgeting portfolio");
    _pcolor = 2;
    xy(t,100*eq_weights[indx_t,2]);

  setwind(2);
    title("\216Tangency portfolio");
    _pcolor = 4;
    xy(t,100*eq_weights[indx_t,3]);

  xtics(0,100,20,2);
  ytics(0,35,5,2);
  asclabel(0,0);

  ylabel("\216Frequency (in %)");
  xlabel("\216Weights (in %)");
  setwind(3);
    title("\216Dynamic risk budgeting portfolio");
    _pbartyp = {6 2}; 
    call histp(100*eq_weights[.,2],20);

  setwind(4);
    title("\216Tangency portfolio");
    _pbartyp = {6 4}; 
    call histp(100*eq_weights[indx_t,3],20);

  graphprt("-c=1 -cf=app2-3-10-3.eps");

  endwind;

print;
print "=====================  Question 3.c  =======================================";

weights1 = miss(zeros(nDates,nAssets),0);
weights2 = miss(zeros(nDates,nAssets),0);

rc1 = miss(zeros(nDates,nAssets),0);
rc2 = miss(zeros(nDates,nAssets),0);

b1 = miss(zeros(nDates,nAssets),0);
b2 = miss(zeros(nDates,nAssets),0);

x0 = ones(nAssets,1)/nAssets;
let b0 = 0.50 0.50;

t = 1;
do until t > nDates;

  cov_t = arraytomat(cov[_max_(t,cov_lags),.,.]);
  short_cov_t = arraytomat(short_cov[_max_(t,short_cov_lags),.,.]);
  er_t = Trends[t,.]';
  sigma_t = sqrt(diag(cov_t));
  mu_t = er_t;
  mu_t = _max_(-0.30,mu_t);
  mu_t = _min_(0.30,mu_t);

  if RD[t] == 1;

    {x,vol,mr,rc,prc} = Compute_ERC_Portfolio(x0,cov_t);
    weights1[t,.] = x';
    rc1[t,.] = prc';
    b1[t,.] = b0';

    b_t = b0 .* (1 + mu_t)^2;
    b_t = b_t / sumc(b_t);

    {x,vol,mr,rc,prc} = Compute_RB_Portfolio(x0,cov_t,b_t);
    weights2[t,.] = x';
    rc2[t,.] = prc';
    b2[t,.] = b_t';

  else;

    x = weights1[t-1,.]';
    {vol,mr,rc,prc} = Compute_Risk_Contribution(x,cov_t);
    weights1[t,.] = x';
    rc1[t,.] = prc';
    b1[t,.] = b1[t-1,.];

    x = weights2[t-1,.]';
    {vol,mr,rc,prc} = Compute_Risk_Contribution(x,cov_t);
    weights2[t,.] = x';
    rc2[t,.] = prc';
    b2[t,.] = b2[t-1,.];

  endif;

  t = t + 1;
endo;

Backtests = zeros(nDates,3);
nBacktests = cols(Backtests);
Backtests[.,1] = Simulate_Monthly_Backtest(Dates,Weights1,Basket,0,0);
Backtests[.,2] = Simulate_Monthly_Backtest(Dates,Weights2,Basket,0,0);
Backtests[.,3] = Backtest_Libor;

t_Daily = selif(seqa(1,1,nDates), Dates .>= 20000101);
Dates = Dates[t_Daily];
Backtests = Backtests[t_Daily,.];
Backtests = 100 * Backtests./Backtests[1,.];
Libor_Index = Backtests[.,cols(Backtests)];
t_RD = RD[t_Daily];
eq_budgets = b1[t_Daily,1]~b2[t_Daily,1];
eq_weights = weights1[t_Daily,1]~weights2[t_Daily,1];

turnover = zeros(nBacktests,1);

i = 1;
do until i > nBacktests-1;
  str = itos("weights",i);
  x = varget(str);
  x = x[t_Daily,.];
  x = selif(x,t_RD);
  turnover[i] = 52*meanc(packr(sumr(abs(x - lag1(x)))));
  i = i + 1;
endo;

indx_t = selif(seqa(1,1,rows(t_Dates)),t_RD);
t = seqa(1,1,rows(indx_t));

graphset;
  window3u;

    _pdate = ""; _pnum = 2; _pframe = 0; 
    _pltype = 3|6|1; _plwidth = 10; _pcolor = 1|2|3;
    _paxht = 0.20; _pnumht = 0.20; _ptitlht = 0.275;

    xtics(1,rows(t),52.08*2,2);
    let xlab = "2000" "02" "04" "06" "08" "10" "12";
    asclabel(xlab,0);

  setwind(1);
    ytics(0,100,20,2);
    xy(t,100*eq_budgets[indx_t,.]);

  setwind(2);
    ytics(0,100,20,2);
    xy(t,100*eq_weights[indx_t,.]);

  setwind(3);
    ytics(80,220,20,2);
    _plegstr = "Risk parity\000Dynamic risk parity\000Fed fund";
    _plegctl = {2 6 1.5 5.25};
    xy(t,backtests[indx_t,1 2]);

    graphprt("-c=1 -cf=app2-3-10-4.eps");
  endwind;

stats = zeros(6,nBacktests);

iter = 1;
do until iter > nBacktests;
  report = Reporting_Backtest(Dates[.,1],Backtests[.,iter],0,Libor_Index,0,0);
  stats[1,iter] = vread(Report,"mu");
  stats[2,iter] = vread(Report,"sigma");
  stats[3,iter] = vread(Report,"SR");
  stats[4,iter] = vread(Report,"MDD");
  stats[5,iter] = vread(Report,"sk");
  stats[6,iter] = vread(Report,"kt")-3.0;
  iter = iter + 1;
endo;

stats[1:2 4,.] = stats[1:2 4,.] * 100;
stats = stats|turnover';
print ftosa(stats',"%lf",3,2);

