/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 363 (Exercise B.3.3)
**
*/

new;
library rpb,pgraph;

cls;

// Procedures

proc (1) = Compute_CC_Covariance(sigma,rho);
  local n,cov;
  n = rows(sigma);
  rho = diagrv(rho*ones(n,n),1.0);
  cov = rho .* sigma .* sigma';
  retp(cov);
endp;

proc (1) = Compute_Rho_Max(sigma);
  local n,rho_max;
  n = rows(sigma);
  rho_max = 1 ./ (sigma^3) .* ( sumc(1./sigma) - (n-1)./sigma );
  rho_max = minc(rho_max);
  retp(rho_max);
endp;

/*
**> Question 1.a
**
*/

print;
print "=====================  Question 1  =======================================";

n = 10;
sigma = rndu(n,1);
rho = 0.325;
cov = Compute_CC_Covariance(sigma,rho);
C = cov ./ sigma ./ sigma';
G = sigma * sigma';
inv_G = 1 ./ G;
e = ones(n,1);
inv_R = (rho*e*e' - ((n-1)*rho+1)*eye(n))/( (n-1)*rho^2-(n-2)*rho-1 );

inv_cov = rho/( (n-1)*rho^2-(n-2)*rho-1 ) .* inv_G;
diag_cov = - ((n-2)*rho+1) / ( (n-1)*rho^2-(n-2)*rho-1 ) ./ (sigma^2);
inv_cov = diagrv(inv_cov,diag_cov);

print ftosa(inv_cov-invpd(cov),"%lf",3,3);

/*
**> Question 1.b
**
*/

print;
print "=====================  Question 1.b  =======================================";

let sigma = 0.20 0.20 0.20 0.20 0.20;
n = rows(sigma);
rho = 0.9999;
cov = Compute_CC_Covariance(sigma,rho);

A = ones(1,n); B = 1; Bounds = (-20~20) .* ones(n,1);

{x,sigma_x,retcode} = Compute_MV_Portfolio(cov,A,B,0,0,Bounds);
y = -((n-1) * rho + 1)./(sigma^2) + rho./sigma .* sumc(1./sigma);
y = y / sumc(y);
z = -n./(sigma^2) + 1 ./ sigma .* sumc(1./sigma);
// z = z / sumc(z);
print ftosa(100*(sigma~x~y~z),"%lf",3,2);

print;
print "=====================  Question 1.d  =======================================";

n = 10;
rho = 0.5;
C = Compute_CC_Covariance(ones(n,1),rho);
det_C = (1-rho)^(n-1) * ((n-1)*rho + 1);
print det_C~det(C);

print;
print "=====================  Question 1.e  =======================================";

n = 5;
sigma = 0.10 + 0.20 * rndu(n,1);
rho = -0.10;
cov = Compute_CC_Covariance(sigma,rho);

{x,sigma_x,retcode} = Compute_MV_Portfolio(cov,A,B,0,0,Bounds);
y = -((n-1) * rho + 1)./(sigma^2) + rho./sigma .* sumc(1./sigma);
y = y / sumc(y);
H = 1 ./ meanc(1./sigma);
z = (n * rho * (1 ./ sigma - 1 ./ H) + (1-rho) ./ sigma) ./ sigma;
print ftosa(100*(sigma~x~y~z~z/sumc(z)),"%lf",3,2);

print;
print "=====================  Question 1.f  =======================================";

nIters = 200000;

print "Set #1         ";
print "---------------";

n = 4;
let sigma = 0.10 0.15 0.20 0.25;

Results = zeros(n,4);

rho_min = -1/(n-1);
rho_max = 0.99;
rho_Vector = seqa(rho_min,(rho_max-rho_min)/(nIters - 1),nIters);

x_min = zeros(nIters,1);
sigma_max = maxc(sigma);
rho_star1 = (1 ./ sigma_max) / (sumc(1 ./ sigma) - (n-1) ./ sigma_max);
rho_star2 = (1 ./ sigma) ./ (sumc(1 ./ sigma) - (n-1) ./ sigma);
rho_star2 = minc(delif(rho_star2, (rho_star2 .< 0) .or (rho_star2 .> 1.0)));
rho_min = -1/(n-1);

rho_star = rho_star1;

Results = zeros(n,6);
iter = 1;
do until iter > 6;
  if iter == 1;    
    rho = rho_min;
  elseif iter == 2;
    rho = 0.0;
  elseif iter == 3;
    rho = rho_star;
  elseif iter == 4;
    rho = 0.90;
  elseif iter == 5;
    rho = 0.999999;
  endif;
  x = -((n-1) * rho + 1)./(sigma^2) + rho./sigma .* sumc(1./sigma);
  x = x / sumc(x); 
  Results[.,iter] = x;
  if iter == 6;
    x = 1./sigma .* (sumc(1./sigma) - n ./ sigma);
    x = x / sumc(x); 
    Results[.,iter] = x;
  endif;
  iter = iter + 1;
endo;

print ftosa(100*Results,"%lf",3,2);
print ftos(100*rho_star,"rho* = %lf",3,2);

iter = 1;
do until iter > nIters;
  rho = rho_Vector[iter];
  x = -((n-1) * rho + 1)./(sigma^2) + rho./sigma .* sumc(1./sigma);
  x = x / sumc(x);
  x_min[iter] = minc(x);
  iter = iter + 1;
endo;

all_rho_star = rho_star;
all_rho = rho_Vector;
all_x = x_min;

indx = sumc(x_min .>= 0.0);
rho_star3 = rho_Vector[indx];
print ftos(100*rho_star1,"rho* = %lf",3,2);
print ftos(100*rho_star2,"rho* = %lf",3,2);
print ftos(100*rho_star3,"rho* = %lf",3,2);

print "Set #2         ";
print "---------------";

n = 5;
let sigma = 0.19 0.20 0.21 0.22 0.20;

rho_min = -1/(n-1);
rho_max = 0.99;
rho_Vector = seqa(rho_min,(rho_max-rho_min)/(nIters - 1),nIters);

x_min = zeros(nIters,1);
sigma_max = maxc(sigma);
rho_star1 = (1 ./ sigma_max) / (sumc(1 ./ sigma) - (n-1) ./ sigma_max);
rho_star2 = (1 ./ sigma) ./ (sumc(1 ./ sigma) - (n-1) ./ sigma);
rho_star2 = minc(delif(rho_star2, (rho_star2 .< 0) .or (rho_star2 .> 1.0)));
rho_min = -1/(n-1);

rho_star = rho_star1;

Results = zeros(n,6);
iter = 1;
do until iter > 6;
  if iter == 1;    
    rho = rho_min;
  elseif iter == 2;
    rho = 0.0;
  elseif iter == 3;
    rho = rho_star;
  elseif iter == 4;
    rho = 0.90;
  elseif iter == 5;
    rho = 0.999999;
  endif;
  x = -((n-1) * rho + 1)./(sigma^2) + rho./sigma .* sumc(1./sigma);
  x = x / sumc(x); 
  Results[.,iter] = x;
  if iter == 6;
    x = 1./sigma .* (sumc(1./sigma) - n ./ sigma);
    x = x / sumc(x); 
    Results[.,iter] = x;
  endif;
  iter = iter + 1;
endo;

print ftosa(100*Results,"%lf",3,2);
print ftos(100*rho_star,"rho* = %lf",3,2);

iter = 1;
do until iter > nIters;
  rho = rho_Vector[iter];
  x = -((n-1) * rho + 1)./(sigma^2) + rho./sigma .* sumc(1./sigma);
  x = x / sumc(x);
  x_min[iter] = minc(x);
  iter = iter + 1;
endo;

all_rho_star = all_rho_star | rho_star;
all_rho = all_rho~rho_Vector;
all_x = all_x~x_min;

indx = sumc(x_min .>= 0.0);
rho_star3 = rho_Vector[indx];
print ftos(100*rho_star1,"rho* = %lf",3,2);
print ftos(100*rho_star2,"rho* = %lf",3,2);
print ftos(100*rho_star3,"rho* = %lf",3,2);

print "Set #3         ";
print "---------------";

n = 6;
let sigma = 0.02 0.20 0.40 0.60 0.80 1.00;

rho_min = -1/(n-1);
rho_max = 0.99;
rho_Vector = seqa(rho_min,(rho_max-rho_min)/(nIters - 1),nIters);

x_min = zeros(nIters,1);
sigma_max = maxc(sigma);
rho_star1 = (1 ./ sigma_max) / (sumc(1 ./ sigma) - (n-1) ./ sigma_max);
rho_star2 = (1 ./ sigma) ./ (sumc(1 ./ sigma) - (n-1) ./ sigma);
rho_star2 = minc(delif(rho_star2, (rho_star2 .< 0) .or (rho_star2 .> 1.0)));
rho_min = -1/(n-1);

rho_star = rho_star1;

Results = zeros(n,6);
iter = 1;
do until iter > 6;
  if iter == 1;    
    rho = rho_min;
  elseif iter == 2;
    rho = 0.0;
  elseif iter == 3;
    rho = rho_star;
  elseif iter == 4;
    rho = 0.90;
  elseif iter == 5;
    rho = 0.999999;
  endif;
  x = -((n-1) * rho + 1)./(sigma^2) + rho./sigma .* sumc(1./sigma);
  x = x / sumc(x); 
  Results[.,iter] = x;
  if iter == 6;
    x = 1./sigma .* (sumc(1./sigma) - n ./ sigma);
    x = x / sumc(x); 
    Results[.,iter] = x;
  endif;
  iter = iter + 1;
endo;

print ftosa(100*Results,"%lf",3,2);
print ftos(100*rho_star,"rho* = %lf",3,2);

iter = 1;
do until iter > nIters;
  rho = rho_Vector[iter];
  x = -((n-1) * rho + 1)./(sigma^2) + rho./sigma .* sumc(1./sigma);
  x = x / sumc(x);
  x_min[iter] = minc(x);
  iter = iter + 1;
endo;

all_rho_star = all_rho_star | rho_star;
all_rho = all_rho~rho_Vector;
all_x = all_x~x_min;

indx = sumc(x_min .>= 0.0);
rho_star3 = rho_Vector[indx];
print ftos(100*rho_star1,"rho* = %lf",3,2);
print ftos(100*rho_star2,"rho* = %lf",3,2);
print ftos(100*rho_star3,"rho* = %lf",3,2);

print ftosa(100*all_rho_star,"%lf",3,2);

e = ones(3,1);
graphset;
  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0; _pcross = 1;
  _pltype = 6|1|3; _plwidth = 10; _pcolor = 1|4|2;
  xlabel("\216\202r\201 (in %)");
  ylabel("\216inf x]i[[*] (in %)");
  xtics(-40,100,20,2);
  ytics(-40,30,10,2);
  _plegstr = "n = 4\000n = 5\000n = 6";
  _plegctl = {2 5 4 1.5};
  graphprt("-c=1 -cf=app2-3-3-1.eps");
  xy(100*all_rho,100*all_x);
 
print;
print "=====================  Question 2.d  =======================================";

n = 6;
let _beta        = 0.30 0.20 0.50 0.50 0.40 0.50;
let sigma_tilde = 0.05 0.10 0.10 0.12 0.05 0.02;

sigma_m = 0.25;
sigma_m_sqr = sigma_m^2;

D = diagrv(zeros(n,n),sigma_tilde^2);
beta_tilde = _beta ./ (sigma_tilde^2);
kappa = _beta'beta_tilde;
invSigma = invpd(D) - sigma_m_sqr/(1+sigma_m_sqr*kappa)*beta_tilde*beta_tilde';
e = ones(n,1);

vol_x = 1/sqrt(e'*invSigma*e);
beta_star = (1 + sigma_m_sqr*kappa)/(sigma_m_sqr * sumc(beta_tilde));
x = (vol_x^2) ./ (sigma_tilde^2) .* (1 - _beta/beta_star);

cov = sigma_m_sqr*_beta*_beta'+D;
{x2,vol2,retcode} = Compute_MV_Portfolio(cov,ones(1,n),1,0,0,-5~5);
print ftosa(100*(x~x2),"%lf",3,2);

n = 6;
_beta = 0.5*ones(n,1);
sigma_tilde = rndu(n,1);

sigma_m = 0.25;
sigma_m_sqr = sigma_m^2;

D = diagrv(zeros(n,n),sigma_tilde^2);
beta_tilde = _beta ./ (sigma_tilde^2);
kappa = _beta'beta_tilde;
invSigma = invpd(D) - sigma_m_sqr/(1+sigma_m_sqr*kappa)*beta_tilde*beta_tilde';
e = ones(n,1);

vol_x = 1/sqrt(e'*invSigma*e);
beta_star = (1 + sigma_m_sqr*kappa)/(sigma_m_sqr * sumc(beta_tilde));
x = (vol_x^2) ./ (sigma_tilde^2) .* (1 - _beta/beta_star);

cov = sigma_m_sqr*_beta*_beta'+D;
{x2,vol2,retcode} = Compute_MV_Portfolio(cov,ones(1,n),1,0,0,-5~5);
print ftosa(100*(x~x2),"%lf",3,2);

n = 6;
_beta = rndu(n,1);
sigma_tilde = 0.05 * ones(n,1);

sigma_m = 0.25;
sigma_m_sqr = sigma_m^2;

D = diagrv(zeros(n,n),sigma_tilde^2);
beta_tilde = _beta ./ (sigma_tilde^2);
kappa = _beta'beta_tilde;
invSigma = invpd(D) - sigma_m_sqr/(1+sigma_m_sqr*kappa)*beta_tilde*beta_tilde';
e = ones(n,1);

vol_x = 1/sqrt(e'*invSigma*e);
beta_star = (1 + sigma_m_sqr*kappa)/(sigma_m_sqr * sumc(beta_tilde));
x = (vol_x^2) ./ (sigma_tilde^2) .* (1 - _beta/beta_star);

cov = sigma_m_sqr*_beta*_beta'+D;
{x2,vol2,retcode} = Compute_MV_Portfolio(cov,ones(1,n),1,0,0,-5~5);
print ftosa(100*(x~x2),"%lf",3,2);

print;
print "=====================  Question 2.e  =======================================";

n = 6;
let _beta        = 0.70 0.80 0.90 1.00 1.20 1.20;
let sigma_tilde = 0.05 0.10 0.10 0.05 0.05 0.20;

let sigma_m[5,1] = 0.05 0.10 0.15 0.20 0.25;

e = ones(n,1);

x1 = zeros(n,5);
x2 = zeros(n,5);
all_beta_star = zeros(5,1);

i = 1;
do until i > 5;
  sigma_m_sqr = sigma_m[i]^2;

  D = diagrv(zeros(n,n),sigma_tilde^2);
  beta_tilde = _beta ./ (sigma_tilde^2);
  kappa = _beta'beta_tilde;
  invSigma = invpd(D) - sigma_m_sqr/(1+sigma_m_sqr*kappa)*beta_tilde*beta_tilde';

  vol_x = 1/sqrt(e'*invSigma*e);
  beta_star = (1 + sigma_m_sqr*kappa)/(sigma_m_sqr * sumc(beta_tilde));
  x1[.,i] = (vol_x^2) ./ (sigma_tilde^2) .* (1 - _beta/beta_star);

  cov = sigma_m_sqr*_beta*_beta'+D;
  {x2[.,i],vol2,retcode} = Compute_MV_Portfolio(cov,ones(1,n),1,0,0,-5~5);

  all_beta_star[i] = beta_star;

  i = i + 1;
endo;

print ftosa(100*sigma_m',"%lf",3,2);
print ftosa(100*x1,"%lf",3,2);
print ftosa(all_beta_star',"%lf",3,2);
print;
print ftosa(100*x2,"%lf",3,2);

