/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 48 (Figure 1.17)
**
*/

new;
library rpb,pgraph;

cls;

let mu =    0.05 0.09 0.07 0.06;
let sigma = 0.04 0.15 0.05 0.10;
let rho =   1.00 
            0.10  1.00
            0.40  0.20  1.00
           -0.10 -0.10 -0.20 1.00;
rho = xpnd(rho);
Cov = rho .* sigma .* sigma';
n = rows(mu);

Bounds = 0~1;

nP = 60;
macheps = 0.00001;

gamma_ = seqa(-1,0.005,300);
{x,er,risk,gamma_x,retcode} = Compute_MVO_Portfolio(mu,cov,1,0,0,0,Bounds,gamma_,0);
er_min = minc(er);
er_max = maxc(er);
er_grid = seqa(er_min+macheps,(er_max-er_min-2*macheps)/(nP-1),nP);
{x_star,er_star,risk_star,gamma_star,retcode} = Compute_MVO_Portfolio(mu,cov,1,0,0,0,Bounds,er_grid,1);

nS = 300;
all_er = zeros(nP,nS);
all_risk = zeros(nP,nS);
all_x = zeros(n,nP);
x0 = zeros(n,nS);

rndseed 123;

indx = 42;

i = 1;
do until i > nS;
  r = rndmn(mu,Cov,100);
  m = meanc(r);
  c = vcx(r);
  {x_S,er_S,risk_S,gamma_S,retcode} = Compute_MVO_Portfolio(m,cov,1,0,0,0,Bounds,gamma_,0);
  er_S_min = minc(er_S);
  er_S_max = maxc(er_S);
  er_S_grid = seqa(er_S_min+macheps,(er_S_max-er_S_min-2*macheps)/(nP-1),nP);
  {x,all_er[.,i],all_risk[.,i],gamma_S,retcode} = Compute_MVO_Portfolio(m,c,1,0,0,0,Bounds,er_S_grid,1);
  all_x = all_x + x;
  x0[.,i] = x[.,indx];
  i = i + 1;
endo;

x_star_Resampling = all_x / nS;
er_star_Resampling = meanc(all_er');
Risk_star_Resampling = meanc(all_Risk');

er_tilde = x_star_Resampling'mu;
Risk_tilde = diag(sqrt(x_star_Resampling'*cov*x_star_Resampling));

er0 = x0'mu;
Risk0 = diag(sqrt(x0'*cov*x0));

Results = x_star|er_star'|risk_star';
print ftosa(100*Results,"%lf",3,2);

Results = x_star_Resampling|er_star_Resampling'|risk_star_Resampling';
print ftosa(100*Results,"%lf",3,2);

all_er = vecr(all_er);
all_risk = vecr(all_risk);
np = rows(all_er);

graphset;
  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6; _plwidth = 10; _pcolor = 1;
  xlabel("\216Volatility (in %)");
  ylabel("\216Expected Return (in %)");
  xtics(3,5.5,0.5,2);
  ytics(5.5,7.5,0.5,2);

  _psym = 0; _pcolor = 0|1|4; _plwidth = 10; _pltype = 6|1|3;
  _plegstr = "Estimated\000Averaged\000Resampled";
  _plegctl = {2 5 5 2};
  _psym = 100*(risk0~er0)~ones(nS,1).*(11~2~3~1~2) |
          100*(Risk_tilde[indx]~er_tilde[indx])~(8~5~4~1~10) ;
  graphprt("-c=1 -cf=chap1-2-3-4.eps");
  xy(100*(Risk_star~Risk_star_Resampling~Risk_tilde),100*(ER_star~ER_star_Resampling~er_tilde));

x0 = meanc(x0');
er0 = x0'mu;
Risk0 = sqrt(x0'cov*x0);

{x1,er1,risk1,gamma1,retcode} = Compute_MVO_Portfolio(mu,cov,1,0,0,0,Bounds,er0,1);
x2 = x_star_Resampling[.,indx];
er2 = x2'mu;
Risk2 = sqrt(x2'cov*x2);

{x1,er1,risk1,gamma1,retcode} = Compute_MVO_Portfolio(mu,cov,1,0,0,0,Bounds,er0,1);
print "-----------------------------------------------------------------";
print ftosa(100*(x0~x1~x2),"%lf",3,2); 
print "-----------------------------------------------------------------";
print ftosa(100*(er0~er1~er2),"%lf",3,2); 
print ftosa(100*(risk0~risk1~risk2),"%lf",3,2);

