/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Pages 117-118 (Shrinkage covariance matrix)
**
*/

new;
library rpb,pgraph;

cls;

let sigma = 0.25 0.15 0.20;
n = rows(sigma);
let rho = 1.0
          0.7  1.0
          0.0 -0.5 1.0;
rho = xpnd(rho);
Sigma = rho .* sigma .* sigma';
call invpd(Sigma);

A = ones(1,n); B = 1; e = ones(n,1);
bounds = (0.0~1.0) .* ones(n,1);

sv = ones(n,1)/n;
{x_GMV,u1,u2,u3,u4,retcode} = Qprog(sv,Sigma,zeros(n,1),A,B,0,0,0);
{x_tilde,lambda0,u2,lambda_minus,lambda_plus,retcode} = Qprog(sv,Sigma,zeros(n,1),A,B,0,0,Bounds);

Sigma_tilde = Sigma + (lambda_plus - lambda_minus) * e' + e * (lambda_plus - lambda_minus)';
call invpd(Sigma_Tilde);

{x_bar,u1,u2,u3,u4,retcode} = QProg(ones(n,1)/n,Sigma_tilde,zeros(n,1),A,B,0,0,0);

results = x_GMV~x_tilde~x_bar;
print ftosa(100*results,"%lf",5,3);

declare matrix _external_delta;
alpha = 4;

proc MinProc(x);
  retp( x'Sigma*x/alpha );
endp;

proc IneqProc(x);
  retp( _external_delta - sumc(abs(x)));
endp;

_external_delta = 1.25;

_sqp_A = A;
_sqp_B = B;
_sqp_C = error(0);
_sqp_D = error(0);
_sqp_Bounds = error(0);
_sqp_DirTol = 1e-6;
_sqp_IneqProc = &IneqProc;
{x0,fmin,Lagrange,retcode} = sqpSolve(&MinProc,sv);
lambda0 = vread(Lagrange,"lineq");
lambda_c = vread(Lagrange,"nlinineq");

d = (x0 .<= 0);
foc = Sigma*x0 - lambda0 - lambda_c .* d;
print foc;

lambda_c = alpha * lambda_c;

Sigma_tilde = Sigma - (lambda_c*d*e' + lambda_c*e*d'); 
call invpd(Sigma_Tilde);

{x_bar,u1,u2,u3,u4,retcode} = QProg(sv,Sigma_tilde,zeros(n,1),A,B,0,0,0);

cls;
results = x_GMV~x0~x_bar;
print ftosa(100*results,"%lf",5,3);