/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 137 (Polico invariance property)
**
*/

new;
library rpb,pgraph;

cls;

__output = 0;
_rpb_algorithm = 2;

let sigma[4,1] = 0.10 0.15 0.20 0.25;
n = rows(sigma);
rho = diagrv(0.20*ones(n,n),1.0);
cov1 = rho .* sigma .* sigma';

bounds = (0~1) .* ones(n,1);
{x,vol_min,retcode} = Compute_MV_Portfolio(cov1,ones(1,n),1,0,0,bounds);
{vol,mr,rc,prc} = Compute_Risk_Contribution(x,cov1);

let c = 0.10 0.10 0.30 0.50;

A = eye(4) | c';
cov2 = A*cov1*A';
sigma = sqrt(diag(cov2));
print ftosa(100*sigma,"%lf",3,2);
rho = corrvc(cov2);
print ftosa(100*rho,"%lf",3,2);

let b1 = 0.40 0.30 0.20 0.10;
b1 = prc;

{x1,vol1,mr1,rc1,prc1} = Compute_RB_Portfolio(0,cov1,b1);

b2 = compute_b_prime(b1,c);
new_b1 = b2[1:4] + c*b2[5];
{x2,vol2,mr2,rc2,prc2} = Compute_RB_Portfolio(0,cov2,b2);

x3 = x2[1:4] + c*x2[5];
{vol3,mr3,rc3,prc3} = Compute_Risk_Contribution(x3,cov1);

cls;

print ftosa(100*vol1,"%lf",3,2);
print ftosa(100*(x1~mr1~rc1~prc1),"%lf",3,2);
print "------------------------------------------------";
print ftosa(100*new_b1,"%lf",3,2);
print "------------------------------------------------";
print ftosa(100*vol2,"%lf",3,2);
print ftosa(100*(x2~mr2~rc2~prc2),"%lf",3,2);
print "------------------------------------------------";
print ftosa(100*vol3,"%lf",3,2);
print ftosa(100*(x3~mr3~rc3~prc3),"%lf",3,2);

proc compute_b_prime(b,c);
  local n,A,b_prime;
  n = rows(b);
  A = (eye(n)~c)|ones(1,n+1);
  B = b | 1;
//  b_prime = b/A;
  b_prime = pinv(A)*b;
  retp(b_prime);
endp;

