/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Pages 139 & 140 (Tables 2.25, 2.26, 2.27 and 2.28 & Figure 2.11)
**
*/

new;
library rpb,pgraph;

__output = 0;

cls;

let sigma[6,1] = 0.20 0.30 0.25 0.15 0.10 0.30;
m = rows(sigma);
rho = diagrv(0.00*ones(m,m),1.0);
Omega = rho .* sigma .* sigma';

let W[3,6] = 0.25 0.25 0.25 0.25 0.00 0.00
             0.00 0.00 0.25 0.25 0.25 0.25
             0.00 0.00 0.50 0.50 0.00 0.00;
n = rows(W);

_Sigma_ = W * Omega * W';
sigma_Index = sqrt(diag(_Sigma_));
rho_Index = _Sigma_ ./ sigma_index ./ sigma_index';

print "sigma (synthetic)  = ";
print ftosa(100*sigma_Index,"%lf",3,2);
print "-----------------------------------------------";

print "rho (synthetic)  = ";
print ftosa(100*rho_Index,"%lf",3,2);
print "-----------------------------------------------";

{x,vol,mr,rc,prc} = Compute_ERC_Portfolio(0,_Sigma_);

all_rc_synthetic = zeros(n,4);
all_rc_primary = zeros(m,4);

let x1 = 0.36 0.38 0.26;
{vol,mr,rc,prc} = Compute_Risk_Contribution(x1,_Sigma_);
print ftosa(100*vol,"%lf",3,2);
print ftosa(100*(x1~mr~rc~prc),"%lf",3,2);
print "-----------------------------------------------";
all_rc_synthetic[.,1] = rc;

y1 = W'x1;
{vol,mr,rc,prc} = Compute_Risk_Contribution(y1,Omega);
print ftosa(100*vol,"%lf",3,2);
print ftosa(100*(y1~mr~rc~prc),"%lf",3,2);
print "-----------------------------------------------";
all_rc_primary[.,1] = rc;

let x2 = 0.48 0.50 0.02;

{vol,mr,rc,prc} = Compute_Risk_Contribution(x2,_Sigma_);
print ftosa(100*vol,"%lf",3,2);
print ftosa(100*(x2~mr~rc~prc),"%lf",3,2);
print "-----------------------------------------------";
all_rc_synthetic[.,2] = rc;

y2 = W'x2;
{vol,mr,rc,prc} = Compute_Risk_Contribution(y2,Omega);
print ftosa(100*vol,"%lf",3,2);
print ftosa(100*(y2~mr~rc~prc),"%lf",3,2);
print "-----------------------------------------------";
all_rc_primary[.,2] = rc;

q = seqa(0,1/100,101);
Lorenz_Curve = zeros(rows(q),4);
Gini = zeros(4,1);

i = 1;
do until i > 4;
  if i == 1;
    tmp = all_rc_synthetic[.,1];
  elseif i == 2;
    tmp = all_rc_synthetic[.,2];
  elseif i == 3;
    tmp = all_rc_primary[.,1];
  elseif i == 4;
    tmp = all_rc_primary[.,2];
  endif;

  tmp = rev(sortc(tmp,1));
  tmp = double(tmp);
  tmp = cumsumc(tmp);
  tmp = tmp/tmp[rows(tmp)];
  x = seqa(1,1,rows(tmp))/rows(tmp);
  tmp = 0 | tmp;
  x = 0 | x;
  
  c = csspline(x,tmp,1,1);
  Lorenz_Curve[.,i] = fspline(c,q);
  Gini[i] = 2*intspline(c,1|0) - 1.0;

  i = i + 1;
endo;

print "Gini = ";
print ftosa(100*Gini,"%lf",3,2);

Lorenz_Curve = _min_(Lorenz_Curve,1.0);
Gini = reshape(Gini,2,2);

graphset;
  window3d;
  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6|1|3; _plwidth = 10; _pcolor = 2|4;
  _paxht = 0.22; _pnumht = 0.20; _ptitlht = 0.275;
  xlabel("\216% of assets");
  ylabel("\216% of RC");
  xtics(0,100,20,2);
  ytics(0,100,20,2);

  _plegstr = "Portfolio #1\000Portfolio #2";

  setwind(1);
    title("\216Synthetic assets");
    xy(100*q,100*Lorenz_Curve[.,1 2]);

  setwind(2);
    title("\216Primary assets");
    _plegctl = {2 6 5 2};
    xy(100*q,100*Lorenz_Curve[.,3 4]);

  xlabel("");
  ylabel("");

  setwind(3);
    xtics(0,3,1,0);
    let string xlab = "Synthetic" "Primary";
    asclabel(xlab,0);
    title("\216Gini coefficient (in %)");
    _pbartyp = {6 2,2 4}; _pbarwid = -0.75;

    _plegctl = {2 6 8 3};
    bar(0,100*Gini);

    graphprt("-c=1 -cf=chap2-5-1-3.eps");

  endwind;

proc (1) = double(x);
  local n,y;

  n = rows(x);
  y = 0.5 * (x + lag1(x));
  y = trimr(y,1,0) | error(0);
  y = reshape(x~y,2*n,1);
  y = packr(y);

  retp(y);
endp;
