/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 185 (Table 3.14)
**
*/

new;
library rpb,pgraph;

cls;

PathName = "data//";
datFileName = Pathname $+ "sx5e-info";
loads info = ^datFileName;
datFileName = Pathname $+ "sx5e-weights";
Composition = loadd(datFileName);
datFileName = Pathname $+ "sx5e-prices";
Prices = loadd(datFileName);
nStocks = cols(Prices) - 2;

Weights = Composition[.,2:cols(Composition)]/100;

Rebalancing_Dates = Composition[.,1];
Rebalancing_Dates[1] = 19920101;
nRD = rows(Rebalancing_Dates);

Dates = Prices[.,1];
Index = Prices[.,2];
Prices = Prices[.,3:2+nStocks];

EndDate = 20120928;

Lag_CovMatrix = 260;
FileName = itos("Results\\sx5e-Weights-basic-",Lag_CovMatrix);
loadm Computed_Weights = ^FileName;

Weights_MV = array3d_get_x(Computed_Weights,1);
Weights_MV_10 = array3d_get_x(Computed_Weights,5);
Weights_MV_5 = array3d_get_x(Computed_Weights,7);

Weights_MDP = array3d_get_x(Computed_Weights,3);
Weights_MDP_10 = array3d_get_x(Computed_Weights,6);
Weights_MDP_5 = array3d_get_x(Computed_Weights,8);

Index_MV = Simulate_Backtest(Rebalancing_Dates,Weights_MV,Dates,Prices,0);
Index_MV_10 = Simulate_Backtest(Rebalancing_Dates,Weights_MV_10,Dates,Prices,0);
Index_MV_5 = Simulate_Backtest(Rebalancing_Dates,Weights_MV_5,Dates,Prices,0);

Index_MDP = Simulate_Backtest(Rebalancing_Dates,Weights_MDP,Dates,Prices,0);
Index_MDP_10 = Simulate_Backtest(Rebalancing_Dates,Weights_MDP_10,Dates,Prices,0);
Index_MDP_5 = Simulate_Backtest(Rebalancing_Dates,Weights_MDP_5,Dates,Prices,0);

Data = Dates~Index~Index_MV~Index_MV_10~Index_MV_5~Index_MDP~Index_MDP_10~Index_MDP_5;
Data = packr(Data);
Data = selif(Data, (Data[.,1] .>= 19921231) .and (Data[.,1] .<= EndDate));

saved_Data = Data;

FileName = "Data\\sx5e-erc-libor";
tmpData = loadd(FileName);
indx = indnv(Data[.,1],tmpData[.,1]);
Data = Data~tmpData[.,5];

Dates = Data[.,1];
Libor_Index = Data[.,9];
nDates = rows(Dates);
Data[.,2] = Data[.,2] ./ Data[1,2] * 1;
Data = 100* Data[.,2:8];

Monthly_Dates = selif(Dates, lag(trunc(Dates/100)) ./= (trunc(Dates/100)));
nMonthly_Dates = rows(Monthly_Dates);

i = 1;
do until i > nMonthly_Dates;
  tmp = selif(Dates, trunc(Monthly_Dates[i]/100) .== trunc(Dates/100 ));
  Monthly_Dates[i] = tmp[rows(tmp)];
  i = i + 1;
endo;

indx = indnv(Monthly_Dates,Dates);
Monthly_Data = Data[indx,.];

Backtests = Data;
Rdt = packr(Backtests./lag1(Backtests) - 1.0);
Rdt_Monthly = packr(Monthly_Data ./lag1(Monthly_Data) - 1.0);

nCols = cols(Backtests);
stats = zeros(12,nCols);
indx_bench = 1;

i = 1;
do until i > nCols;
  Report = Reporting_Backtest(Dates,Backtests[.,i],Backtests[.,indx_bench],Libor_Index,0,0);
  mu = vread(Report,"mu");
  sigma = vread(Report,"sigma");
  sh = vread(Report,"SR");
  te = vread(Report,"sigma_te");
  ir = vread(Report,"ir");
  dd = vread(Report,"mdd");
  rho = submat(corrx(Rdt[.,indx_bench~i]),1,2);
  beta = rho * sigma / vread(Report,"sigma_B");

  sk_D = Estimate_Skewness(Rdt[.,i]);
  kt_D = Estimate_Kurtosis(Rdt[.,i]) - 3.0;

  sk_M = Estimate_Skewness(Rdt_Monthly[.,i]); 
  kt_M = Estimate_Kurtosis(Rdt_Monthly[.,i]) - 3.0;

  stats[.,i] = 100*mu|100*sigma|sh|100*te|ir|100*dd|sk_D|kt_D|sk_M|kt_M|100*rho|beta;
  i = i + 1;
endo;

print ftosa(stats[.,2:7],"%lf",3,2);