/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 187 (Table 3.15)
**
*/

new;
library rpb,pgraph;

cls;

PathName = "data//";
datFileName = Pathname $+ "sx5e-info";
loads info = ^datFileName;
datFileName = Pathname $+ "sx5e-weights";
Composition = loadd(datFileName);
datFileName = Pathname $+ "sx5e-prices";
Prices = loadd(datFileName);
nStocks = cols(Prices) - 2;

Weights = Composition[.,2:cols(Composition)]/100;

EndDate = 20120928;

Rebalancing_Dates = Composition[.,1];
Rebalancing_Dates[1] = 19920101;
nRD = rows(Rebalancing_Dates);

Data_Prices = Prices;
Index_Weights = Weights;

let indx_cov = 1 2 6 4 7; // EMP, CC, PCA, Shrinkage-CC Shrinkage-PCA
nCovMethods = rows(indx_cov);

all_stats = {};

iteration = 1;
do until iteration > nCovMethods; 
  iter = indx_cov[iteration];
  
  Dates = Data_Prices[.,1];
  Index = Data_Prices[.,2];
  Prices = Data_Prices[.,3:2+nStocks];

  Lag_CovMatrix = 260;
  w_FileName = itos("Results\\sx5e-Weights-basic-",Lag_CovMatrix) $+ itos("-cov-",iter);
  rc_FileName = itos("Results\\sx5e-RC-basic-",Lag_CovMatrix) $+ itos("-cov-",iter);
  info_FileName = itos("Results\\sx5e-info-basic-",Lag_CovMatrix) $+ "-info";

  loadm Computed_Weights = ^w_FileName;
  loadm Computed_RC = ^rc_FileName;

  Weights_MV = array3d_get_x(Computed_Weights,1);
  Weights_ERC = array3d_get_x(Computed_Weights,2);
  Weights_MDP = array3d_get_x(Computed_Weights,3);
  Weights_EW = array3d_get_x(Computed_Weights,4);
  Weights_MV_10 = array3d_get_x(Computed_Weights,5);
  Weights_MDP_10 = array3d_get_x(Computed_Weights,6);
  Weights_MV_5 = array3d_get_x(Computed_Weights,7);
  Weights_MDP_5 = array3d_get_x(Computed_Weights,8);

  nAW = 9;
  Turnover = zeros(nAW,1);
  Gini = zeros(nAW,2);

  i = 1;
  do until i > nAW;
    Weights = array3d_get_x(Computed_Weights,i);
    Weights = selif(Weights,Rebalancing_Dates.<=EndDate);
    Weights = abs(Weights - lag1(Weights));
    if EndDate > 20091231;
      Weights = delif(Weights,Rebalancing_Dates .== 20100129);
    endif;
    Weights = packr(Weights);
    Turnover[i] = 12*meanc(sumr(Weights));
    i = i + 1;
  endo;

  Turnover = rotater(Turnover',1)';
  
  nRD = sumc(Rebalancing_Dates.<=EndDate);
  
  G1 = miss(zeros(nRD,nAW),0);
  G2 = miss(zeros(nRD,nAW),0);

  i = 13;
  do until i > nRD;

    tmp_Computed_Weights = array3d_get_y(Computed_Weights,i)';
    tmp_Index_Weights = Index_Weights[i,.]';
    tmp_Weights = tmp_Computed_Weights~tmp_Index_Weights;
    indx = selif(seqa(1,1,nStocks),tmp_Index_Weights[.,1] ./= 0.0);
    tmp_Weights = tmp_Weights[indx,.];

    tmp_Computed_RC = array3d_get_y(Computed_RC,i)';
    tmp_RC = tmp_Computed_RC;
    tmp_RC = tmp_RC[indx,.];

    G = zeros(nAW,2);

    j = 1;
    do until j > nAW;

      tmp = tmp_Weights[.,j];
      if ismiss(tmp);
        j = j + 1;
         continue;
      endif;

      tmp = rev(sortc(tmp,1));
      tmp = cumsumc(tmp);
      tmp = tmp/tmp[rows(tmp)];
      x = seqa(0,1,rows(tmp)+1)/rows(tmp);
      tmp = 0|tmp;
      c = csspline(x,tmp,1,1);

      G[j,1] = 2*intspline(c,1|0) - 1.0;

      tmp = tmp_RC[.,j];
      tmp = rev(sortc(tmp,1));
      tmp = cumsumc(tmp);
      x = seqa(0,1,rows(tmp)+1)/rows(tmp);
      tmp = 0|tmp;
      c = csspline(x,tmp,1,1);

      G[j,2] = 2*intspline(c,1|0) - 1.0;
      j = j + 1;
    endo; 

    G1[i,.] = G[.,1]';
    G2[i,.] = G[.,2]';

    i = i + 1;
  endo;

  Gini[.,1] = meanc(packr(G1));
  Gini[.,2] = meanc(packr(G2));

  Gini = rotater(Gini',1)';

  Index_MV = Simulate_Backtest(Rebalancing_Dates,Weights_MV,Dates,Prices,0);
  Index_ERC = Simulate_Backtest(Rebalancing_Dates,Weights_ERC,Dates,Prices,0);
  Index_MDP = Simulate_Backtest(Rebalancing_Dates,Weights_MDP,Dates,Prices,0);
  Index_EW = Simulate_Backtest(Rebalancing_Dates,Weights_EW,Dates,Prices,0);
  Index_MV_10 = Simulate_Backtest(Rebalancing_Dates,Weights_MV_10,Dates,Prices,0);
  Index_MDP_10 = Simulate_Backtest(Rebalancing_Dates,Weights_MDP_10,Dates,Prices,0);
  Index_MV_5 = Simulate_Backtest(Rebalancing_Dates,Weights_MV_5,Dates,Prices,0);
  Index_MDP_5 = Simulate_Backtest(Rebalancing_Dates,Weights_MDP_5,Dates,Prices,0);

  Data = Dates~Index~Index_MV~Index_ERC~Index_MDP~Index_EW~Index_MV_10~Index_MDP_10~Index_MV_5~Index_MDP_5;
  Data = packr(Data);
  Data = selif(Data, (Data[.,1] .>= 19921231) .and (Data[.,1] .<= EndDate));

  Dates = Data[.,1];
  nDates = rows(Dates);
  Data[.,2] = Data[.,2] ./ Data[1,2] * 1;
  Data = 100* Data[.,2:cols(data)];

  dt = etdays2(Dates[1],Dates[rows(Dates)])/365;
  B_Index = Data[.,1];
  alpha = ((Data[nDates,.]./Data[1,.])/(B_Index[nDates]/B_Index[1]))^(1/dt) - 1; 

  Rdt = Data ./lag1(Data) - 1.0;
  Rdt = packr(Rdt);
  e = Rdt - Rdt[.,1];
  te = stdc(e)*sqrt(260); 
  IR = alpha ./ te';

  Turnover = round(100*Turnover);

  Stats = Turnover' |
          Gini' |
          IR ;
  Stats = Stats[.,2:9];

  all_stats = all_Stats|Stats;
  
  str = ftosa(Stats,"%lf",2,2);
  str[1,.] = ftosa(Stats[1,.],"%lf",2,0);

  print /flush str;
  print "-------------------------------------------------------------------------------";
  
  iteration = iteration + 1;
endo;

str = ftosa(all_Stats,"%lf",2,2);
str[seqa(1,4,nCovMethods),.] = ftosa(all_Stats[seqa(1,4,nCovMethods),.],"%lf",2,0);
str = str[.,4 2 1 5 7 3 6 8];
print str;
