/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 206 (Table 4.4 & Figure 4.6)
**
*/

new;
library rpb,pgraph;

cls;

_Bond_Continuous_Time = 1;
_bond_window = 2;

loadm maturity = data\zc-tau;

datFileName = "data\\zc-ir-2009.dat";
data = loadd(datFileName);
data = selif(data,(data[.,1] .>= 20100630) .and (data[.,1] .<= 20120630));
Dates = data[.,1];
nDates = rows(Dates);
R = data[.,2:cols(data)]/100;
tau = seqa(1,1,5);
indx = indnv(tau,Maturity);
R = R[.,indx];

rndseed 123456;
t = seqa(0,1/260,nDates);
Sp = simulate_GBM(1,0.50,0.40,t,1)/100;

n = rows(R);
Rt = R[n,.]';

data = R~Sp;

graphset;
  begwind;
  window(2,3,1);
  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 3; _plwidth = 10; _pcolor = 1;
  _pnumht = 0.24; _paxht = 0.20; _ptitlht = 0.45;

  xtics(0,nDates,262/2,2);
  let string xlab = "Jun. 10" "Dec. 10"  "Jun. 11" "Dec. 11" "Jun. 12";
  let string xlab = "07/10" "12/10"  "07/11" "12/11" "07/12";
  asclabel(xlab,0);

  ytics(0,2.5,0.5,2);
  i = 1;
  do until i > 6;
    _pcolor = i;
    if i == 1;
      title("\216R]t[(1)");
    elseif i == 2;
      title("\216R]t[(2)");
    elseif i == 3;
      title("\216R]t[(3)");
    elseif i == 4;
      title("\216R]t[(4)");
    elseif i == 5;
      title("\216R]t[(5)");
    elseif i == 6;
      title("\216\202l\201");
      ytics(0,5,1,2);
    endif;
    setwind(i);
      xy(0,100*data[.,i]);
    i = i + 1;
  endo;

  graphprt("-c=1 -cf=chap4-2-3-1.eps");

  endwind;

Nominal = 1e6;
Coupon = 5;
Notional = 100;
Maturity = 5;
Ct = Coupon * ones(Maturity,1);
Ct[Maturity] = Ct[Maturity] + Notional;
t = seqa(1,1,Maturity);

P0 = Bond_Price(t,Ct,Rt);
nB = Nominal/P0;
nB = round(nB);

dR = R - lag1(R);
{gVaR1,gVaR2,hVaR1,hVaR2,hVaR3,ES,results} = Bond_VaR(t,Ct,Rt,dR,0.99,1/260);

Results = Results .* (1~1~100~1~1~1);

print ftos(P0,"Bond Price = %lf",5,2);


print;
print ftosa(Results,"%lf",3,3);
print;
print "---------------------------------------------------------------";

print ftos(nB*gVaR1,"Analytical VaR                            = %lf",5,2);
print ftos(nB*gVaR2,"Analytical VaR (without mean)             = %lf",5,2);
print ftos(nB*hVaR1,"Historical VaR                            = %lf",5,2);
print ftos(nB*hVaR2,"Historical VaR (with window smoothing)    = %lf",5,2);
print ftos(nB*hVaR3,"Historical VaR (with elliptic correction) = %lf",5,2);
print ftos(nB*ES,   "Expected Shortfall                        = %lf",5,2);
print "---------------------------------------------------------------";

{gVaR1,gVaR2,hVaR1,hVaR2,hVaR3,ES} = Bond_Portfolio_VaR(nB,t,Ct,Rt,dR,0.99,1/260,0);
print ftos(gVaR1,"Analytical VaR                            = %lf",5,2);
print ftos(gVaR2,"Analytical VaR (without mean)             = %lf",5,2);
print ftos(hVaR1,"Historical VaR                            = %lf",5,2);
print ftos(hVaR2,"Historical VaR (with window smoothing)    = %lf",5,2);
print ftos(hVaR3,"Historical VaR (with elliptic correction) = %lf",5,2);
print ftos(ES,   "Expected Shortfall                        = %lf",5,2);
print "---------------------------------------------------------------";

{gVaR1,gVaR2,hVaR1,hVaR2,hVaR3,ES} = Bond_Portfolio_VaR(nB,t,Ct,Rt,dR,0.99,1/260,1);
print ftos(gVaR1,"Analytical VaR                            = %lf",5,2);
print ftos(gVaR2,"Analytical VaR (without mean)             = %lf",5,2);
print ftos(hVaR1,"Historical VaR                            = %lf",5,2);
print ftos(hVaR2,"Historical VaR (with window smoothing)    = %lf",5,2);
print ftos(hVaR3,"Historical VaR (with elliptic correction) = %lf",5,2);
print ftos(ES,   "Expected Shortfall                        = %lf",5,2);
print "---------------------------------------------------------------";
