/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 229 (RC of the EGBI portfolio)
**
*/

new;
library rpb,pgraph;

cls;

FileName = "Data//sovereign-bonds.fmt";
loadm databuf = ^FileName;
Dates = vread(databuf,"dates");
Spread = vread(databuf,"spread");
Spread_Greece = vread(databuf,"sp-gr");
Duration = vread(databuf,"duration");
Weights_EGBI = vread(databuf,"w(egbi)");
Weights_GDP = vread(databuf,"w(gdp)");
Weights_DEBT = vread(databuf,"w(debt)");
EGBI_index = vread(databuf,"index");
EGBI_Subindex = vread(databuf,"subindex");

FileName = "Results//sovereign-bonds-vol";
loadm sigma_S = ^FileName;

FileName = "Results//sovereign-bonds-rho";
loadm rho = ^FileName;

/*
**> EGBI
**
*/

nCountries = cols(Spread);

Begin_Date = 20071231;
End_Date = 20120630;
nDates = rows(Dates);
indx_Begin_Date = closest_index(Begin_Date,Dates);

sigma_c = Duration .* sigma_S .* Spread;

RC = miss(zeros(nDates,nCountries),0);
Weights = Weights_EGBI;
VOL = miss(zeros(nDates,1),0);

t = indx_Begin_Date;
do until t > nDates;
  x_t = Weights[t,.]';
  rho_t = arraytomat(rho[t,.,.]);
  cov_t = rho_t .* sigma_c[t,.] .* sigma_c[t,.]';
  {vol_t,mr_t,rc_t,prc_t} = Compute_Risk_Contribution(x_t,cov_t);
  RC[t,.] = prc_t';
  VOL[t] = vol_t;
  t = t + 1;
endo;

data = dates~Weights~RC~VOL;
data = selif(data,(data[.,1] .> Begin_Date) .and (data[.,1] .<= End_Date));
FileName = "Results\\sovereign-bonds-egbi";
save ^FileName = data;

